@@ -1,6 +1,6 @@
 ;;; org-faces.el --- Face definitions -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2004-2016 Free Software Foundation, Inc.
+;; Copyright (C) 2004-2017 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <carsten at orgmode dot org>
 ;; Keywords: outlines, hypermedia, calendar, wp
@@ -19,7 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;;; Commentary:
@@ -291,7 +291,7 @@ determines if it is a foreground or a background color."
 (defcustom org-priority-faces nil
   "Faces for specific Priorities.
 This is a list of cons cells, with priority character in the car
-and faces in the cdr.  The face can be a symbol, a color as
+and faces in the cdr.  The face can be a symbol, a color
 as a string, or a property list of attributes, like
     (:foreground \"blue\" :weight bold :underline t).
 If it is a color string, the variable `org-faces-easy-properties'
@@ -398,7 +398,7 @@ follows a #+DATE:, #+AUTHOR: or #+EMAIL: keyword."
 For source-blocks `org-src-block-faces' takes precedence.
 See also `org-fontify-quote-and-verse-blocks'."
   :group 'org-faces
-  :version "25.2")
+  :version "26.1")
 
 (defface org-block-begin-line '((t (:inherit org-meta-line)))
   "Face used for the line delimiting the begin of source blocks."
@@ -558,7 +558,11 @@ month and 365.24 days for a year)."
   :group 'org-faces)
 
 (defface org-agenda-filter-category '((t :inherit mode-line))
-  "Face for categories(s) in the mode-line when filtering the agenda."
+  "Face for categories in the mode-line when filtering the agenda."
+  :group 'org-faces)
+
+(defface org-agenda-filter-effort '((t :inherit mode-line))
+  "Face for effort in the mode-line when filtering the agenda."
   :group 'org-faces)
 
 (defface org-time-grid	   ;Copied from `font-lock-variable-name-face'
